@@ -24,9 +24,11 @@ use crate::glbackend::types::*;
 
 pub mod types
 {
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub type GLenum = u32;
     pub type GLuint = u32;
     pub type GLint = i32;
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub type GLchar = std::os::raw::c_char;
     pub type GLsizei = i32;
 
@@ -50,6 +52,8 @@ pub mod constants
 
     pub const GL_BLEND: GLenum = glow::BLEND;
 
+    pub const GL_SCISSOR_TEST: GLenum = glow::SCISSOR_TEST;
+
     pub const GL_SRC_ALPHA: GLenum = glow::SRC_ALPHA;
     pub const GL_ONE_MINUS_SRC_ALPHA: GLenum = glow::ONE_MINUS_SRC_ALPHA;
 
@@ -61,7 +65,9 @@ pub mod constants
 
     pub const GL_DYNAMIC_DRAW: GLenum = glow::DYNAMIC_DRAW;
 
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub const GL_FALSE: u8 = glow::FALSE;
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub const GL_TRUE: u8 = glow::TRUE;
 
     pub const GL_FLOAT: GLenum = glow::FLOAT;
@@ -98,7 +104,9 @@ pub mod constants
     pub const GL_VERTEX_SHADER: GLenum = glow::VERTEX_SHADER;
     pub const GL_FRAGMENT_SHADER: GLenum = glow::FRAGMENT_SHADER;
 
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub const GL_LINK_STATUS: GLenum = glow::LINK_STATUS;
+    #[cfg_attr(target_arch = "wasm32", allow(dead_code))]
     pub const GL_COMPILE_STATUS: GLenum = glow::COMPILE_STATUS;
     #[allow(dead_code)]
     pub const GL_INFO_LOG_LENGTH: GLenum = glow::INFO_LOG_LENGTH;
@@ -179,6 +187,7 @@ pub trait GLBackend
     unsafe fn gl_enable_debug_message_callback(&self);
     unsafe fn gl_get_string(&self, parameter: GLenum) -> String;
     unsafe fn gl_viewport(&self, x: i32, y: i32, width: i32, height: i32);
+    unsafe fn gl_scissor(&self, x: GLint, y: GLint, width: GLsizei, height: GLsizei);
     unsafe fn gl_pixel_store_i(&self, param: GLenum, value: GLint);
 
     unsafe fn gl_vertex_attrib_pointer_f32(
@@ -483,6 +492,11 @@ impl GLBackend for GLBackendGlow
         self.context.viewport(x, y, width, height)
     }
 
+    unsafe fn gl_scissor(&self, x: GLint, y: GLint, width: GLsizei, height: GLsizei)
+    {
+        self.context.scissor(x, y, width, height);
+    }
+
     unsafe fn gl_pixel_store_i(&self, param: u32, value: i32)
     {
         self.context.pixel_store_i32(param, value)
@@ -848,6 +862,11 @@ impl GLBackend for GLBackendGLRS
         gl::Viewport(x, y, width, height)
     }
 
+    unsafe fn gl_scissor(&self, x: GLint, y: GLint, width: GLsizei, height: GLsizei)
+    {
+        gl::Scissor(x, y, width, height);
+    }
+
     unsafe fn gl_pixel_store_i(&self, param: u32, value: i32)
     {
         gl::PixelStorei(param, value)
