@@ -248,6 +248,17 @@ pub trait WindowHandler<UserEventType = ()>
     {
     }
 
+    /// Invoked when the mouse wheel moves.
+    #[allow(unused_variables)]
+    #[inline]
+    fn on_mouse_wheel_scroll(
+        &mut self,
+        helper: &mut WindowHelper<UserEventType>,
+        distance: MouseScrollDistance
+    )
+    {
+    }
+
     /// Invoked when a keyboard key is pressed.
     ///
     /// To detect when a character is typed, see the
@@ -428,6 +439,16 @@ where
         self.window_handler.on_mouse_button_up(helper, button)
     }
 
+    #[inline]
+    pub fn on_mouse_wheel_scroll(
+        &mut self,
+        helper: &mut WindowHelper<UserEventType>,
+        distance: MouseScrollDistance
+    )
+    {
+        self.window_handler.on_mouse_wheel_scroll(helper, distance)
+    }
+
     #[inline]
     pub fn on_key_down(
         &mut self,
@@ -700,6 +721,54 @@ pub enum MouseButton
     Other(u16)
 }
 
+/// Describes a difference in the mouse scroll wheel position.
+#[derive(Debug, PartialEq, Clone, Copy)]
+pub enum MouseScrollDistance
+{
+    /// Number of lines or rows to scroll in each direction. The `y` field
+    /// represents the vertical scroll direction on a typical mouse wheel.
+    Lines
+    {
+        /// The horizontal scroll distance. Negative values indicate scrolling
+        /// left, and positive values indicate scrolling right.
+        x: f64,
+        /// The vertical scroll distance. Negative values indicate scrolling
+        /// down, and positive values indicate scrolling up.
+        y: f64,
+        /// The forward/backward scroll distance, supported on some 3D mice.
+        z: f64
+    },
+    /// Number of pixels to scroll in each direction. Scroll events are
+    /// expressed in pixels if supported by the device (eg. a touchpad) and
+    /// platform. The `y` field represents the vertical scroll direction on a
+    /// typical mouse wheel.
+    Pixels
+    {
+        /// The horizontal scroll distance. Negative values indicate scrolling
+        /// left, and positive values indicate scrolling right.
+        x: f64,
+        /// The vertical scroll distance. Negative values indicate scrolling
+        /// down, and positive values indicate scrolling up.
+        y: f64,
+        /// The forward/backward scroll distance, supported on some 3D mice.
+        z: f64
+    },
+    /// Number of pages to scroll in each direction (only supported for
+    /// WebCanvas). The `y` field represents the vertical scroll direction on a
+    /// typical mouse wheel.
+    Pages
+    {
+        /// The horizontal scroll distance. Negative values indicate scrolling
+        /// left, and positive values indicate scrolling right.
+        x: f64,
+        /// The vertical scroll distance. Negative values indicate scrolling
+        /// down, and positive values indicate scrolling up.
+        y: f64,
+        /// The forward/backward scroll distance, supported on some 3D mice.
+        z: f64
+    }
+}
+
 /// A virtual key code.
 #[allow(missing_docs)]
 #[derive(Debug, Hash, Ord, PartialOrd, PartialEq, Eq, Clone, Copy)]
