@@ -14,6 +14,7 @@
  *  limitations under the License.
  */
 
+#[deny(warnings)]
 #[cfg(not(all(target_arch = "x86_64", target_os = "linux")))]
 compile_error!("The automated tests currently support Linux x86_64 only");
 
@@ -27,7 +28,7 @@ use speedy2d::color::Color;
 use speedy2d::dimen::Vector2;
 use speedy2d::font::{Font, TextAlignment, TextLayout, TextOptions};
 use speedy2d::image::{ImageDataType, ImageSmoothingMode};
-use speedy2d::shape::Rectangle;
+use speedy2d::shape::{Polygon, Rectangle};
 use speedy2d::GLRenderer;
 
 const NOTO_SANS_REGULAR_BYTES: &[u8] =
@@ -869,6 +870,83 @@ fn main()
         })
     });
 
+    tests.push(GLTest {
+        width: 500,
+        height: 500,
+        name: "polygon_test_1".to_string(),
+        action: Box::new(|renderer| {
+            renderer.draw_frame(|graphics| {
+                graphics.clear_screen(Color::WHITE);
+                let poly = Polygon::new(&[
+                    (100.0, 100.0),
+                    (250.0, 50.0),
+                    (400.0, 100.0),
+                    (300.0, 400.0),
+                    (100.0, 400.0)
+                ]);
+                graphics.draw_polygon(&poly, (0.0, 0.0), Color::RED);
+            });
+        })
+    });
+
+    tests.push(GLTest {
+        width: 500,
+        height: 500,
+        name: "polygon_test_anticlockwise".to_string(),
+        action: Box::new(|renderer| {
+            renderer.draw_frame(|graphics| {
+                graphics.clear_screen(Color::WHITE);
+                let poly = Polygon::new(&[
+                    (100.0, 400.0),
+                    (300.0, 400.0),
+                    (400.0, 100.0),
+                    (250.0, 50.0),
+                    (100.0, 100.0)
+                ]);
+                graphics.draw_polygon(&poly, (0.0, 0.0), Color::RED);
+            });
+        })
+    });
+
+    tests.push(GLTest {
+        width: 500,
+        height: 500,
+        name: "polygon_test_offset".to_string(),
+        action: Box::new(|renderer| {
+            renderer.draw_frame(|graphics| {
+                graphics.clear_screen(Color::WHITE);
+                let poly = Polygon::new(&[
+                    (100.0, 400.0),
+                    (300.0, 400.0),
+                    (400.0, 100.0),
+                    (250.0, 50.0),
+                    (100.0, 100.0)
+                ]);
+                graphics.draw_polygon(&poly, (0.0, 0.0), Color::RED);
+                graphics.draw_polygon(&poly, (-10.0, 20.0), Color::BLUE);
+            });
+        })
+    });
+
+    tests.push(GLTest {
+        width: 500,
+        height: 500,
+        name: "polygon_test_concave_anticlockwise".to_string(),
+        action: Box::new(|renderer| {
+            renderer.draw_frame(|graphics| {
+                graphics.clear_screen(Color::WHITE);
+                let poly = Polygon::new(&[
+                    (100.0, 400.0),
+                    (300.0, 400.0),
+                    (400.0, 100.0),
+                    (250.0, 50.0),
+                    (250.0, 350.0)
+                ]);
+                graphics.draw_polygon(&poly, (0.0, 0.0), Color::RED);
+            });
+        })
+    });
+
     for test in tests {
         log::info!("Running test {}", test.name);
 
